/****************************************************************************
 * WHAM windowing system for Allegro & DJGPP                                *
 * Copyright 1997 Martijn Versteegh                                         *
 * WHAM is swapware                                                         *
 * that means if you use it you must send me something in return            *
 * If you use WHAM in a commercial product, 'something' should preferrable  *
 * be money. A program you made, a postcard or something like that will do  *
 * in most other cases.                                                     *
 *  for contact info see contact.txt in the docs directory.                 *
 *                                                                          *
 *  Disclaimer:                                                             *
 *      I am not responsible for any effect this software may have on you,  *
 *      your properties or anything else connected to you.                  *
 *      Use it at your own risk.                                            *
 ****************************************************************************/

#ifndef _WOBJECT_H_
#define _WOBJECT_H_
#include <string.h>
#include "allegro.h"
#include "winsys.h"
#include "boxlist.h"
#include "geoinfo.h"
#include "topinc.h"
#define  WINDOW_UPPER_BORDER 0
#define  WINDOW_LOWER_BORDER 1
#define  WINDOW_RIGHT_BORDER 2
#define  WINDOW_LEFT_BORDER 3
#define  get_geometric_info get_geoinfo
#define  WINDOW_UL_CORNER 0
#define  WINDOW_UR_CORNER 1
#define  WINDOW_LL_CORNER 2
#define  WINDOW_LR_CORNER 3


#define W_REQUEST_UPDATE 1
#define W_REQUEST_SHOW 2
#define W_REQUEST_HIDE 3
#define W_REQUEST_DELETE 4

class Stack;

class Wobject
{
    public:
        friend class WinSystem;
        friend class Boxlist;
        // constructor destructor
        Wobject(char *name,int size_x,int size_y,int pos_x,int pos_y);
        Wobject(char *name,int size_x,int size_y,int pos_x,int pos_y, WinSystem *its_manager);
        Wobject(char *name,int size_x,int size_y,int pos_x,int pos_y,int (*draw_border)(BITMAP *onto,Wobject *the_object,int from_x,int from_y,int to_x,int to_y));
        Wobject(char *name,int size_x,int size_y,int pos_x,int pos_y, WinSystem *its_manager,int (*draw_border)(BITMAP *onto,Wobject *the_object,int from_x,int from_y,int to_x,int to_y));
        
        //by declaring the destructor virtual, the last destructor in the row of
        // derivations is called, even if it is destroyed via a base pointer
        // it does NOT mean this destructor will not get called for a derived 
        // class , it will only be as if you deleted a pointer to the derived object 
        // instead of a base pointer ( so all destructors are called, in the right order)
        virtual ~Wobject();

        // info functions
        GeoInfo const *get_geoinfo();
        int get_tag();
        // delayed operations, carried out by WinSys in the next loop
        void request(int request); // let Wham_Secretar update this window when it's safe to do so
        								 // for use in dialogfunctions, because alguicontainer claims the
                               // screen pointer for it's own use
                               // and also for letting a window destoy itself

        // showing /hiding
        void update();  // conditionally calls show (only if the object is already visible)
        void update(int xfrom,int yfrom, int xto, int yto);  // conditionally calls show (only if the object is already visible)
        int show();    // shows the object
        int hide();    // hides the object
        int refresh_border();

        // raise/ lower
        int raise();   // makes this the front object
        int lower();
        int raise_and_lock();   // makes this the front object
                                // this function makes it always the total front object, before other locked objects
        
        //  a locked object can not be raised, and when it's on top
        // it will stay on top, unless all objects are locked,
        // and a new one is created, for then this new one will
        // be created on top of the rest 

        // locking functions
        void set_lock(int lock);
        void set_lock_position(int lock); // a position_locked object cannot be moved
        void set_lock_size(int lock);
        void set_lock_scale(int lock);
        void set_lock_viewport_position(int lock);
        void set_lock_viewport_size(int lock);
        void set_lock_visibility(int lock);
        void set_lock_shift(int lock);  // wont be shifted by the shift_all function

        
        // return various objects
        virtual BITMAP *bmp(); // derived classes must defines this one, let it return 0 if it's not allowed to draw on
        WinSystem *get_manager();

        // moving resizeing etc
        int move(int new_pos_x, int new_pos_y);
        int rescale(int nw_size_x, int nw_size_y);
        int move_rescale(int new_pos_x, int new_pos_y,int new_size_x,int new_size_y);
        virtual int resize(int nw_size_x, int nw_size_y);
        // viewport /user area
        virtual void set_user_area(int pos_x, int pos_y, int size_x, int size_y);
        int resize_viewport(int nw_size_x,int nw_size_y);
        int resize_s_viewport(int nw_size_x,int nw_size_y);
        int move_viewport(int nw_x,int nw_y);

        // coordinate helper function
        void transform_coordinates(int *x,int *y);

        // for use in derived classes, to pass information to the mouse hanler etc
        // (the mouse handler & border drawer only see a Wobject, so they cannot reach
        // the child class 's variables)
        int magic_number;
        void *user_data;

        // react to mouse/keyboard / mesage
        virtual int process_keyboard();
        virtual Wobject *process_mouse(int *x = NULL, int *y = NULL);
        virtual int process_message(Wobject *from, int message);
        virtual int inquire_mouse(int x, int y);      // asks the object if the give mouse coordinates(relative to the window) are part of the window

        // attachment to another object
        void attach_to(Wobject *object,int corner_this, int cormer_that, int offset_x, int offset_y);
        
        // get notified when the object is destroyed
        void set_destruction_callback(void (*callback)(Wobject *w));
        

        // transparent or not?
        int see_through;

        // low level
        void notify_children(); //should be called whenever the attached bitmap changes in size
        void freeze();
        void thaw();
        int is_frozen();
        void register_as_property(Wobject *w);
        void refresh_border_info();
        
    private:                                                                        
        int (*Wobject::draw_border)(BITMAP *onto,Wobject *the_object,int from_x,int from_y,int to_x,int to_y);
        GeoInfo geoinfo;
        static int count;               // increases each time a new object is made & decreases when one is closed
        int is_being_deleted;               // used to prevent child managers from call ing this window's draw function if it doesn't exist anymore
        void create(char *name,int size_x,int size_y,int pos_x, int pos_y); // creates a object with bitmap as buffer
        void destroy();                                // just does what it says

        int the_request;


        //these are set to its_manager->Minimum_size_? by create() unless the created sizes were smaller, the they're set to that
        int minimum_size_x;
        int minimum_size_y;


        // draws (a part of) the object the coordinates are screen coordinates
        // pure virtual: derived classes should provide it
        virtual void draw(BITMAP *onto, int x_from, int y_from, int x_to, int y_to) = 0;

        // not pure virtual, but implementation in Wobject doesn't do anything, so if you need this
        // function the derived classes sould redefine it
        virtual void point(BITMAP *onto,int x,int y);

        void draw_shadow(BITMAP *onto, int x_from, int y_from, int x_to, int y_to); // draws (a part of) the shadow the coordinates are screen coordinates
        void remove_shadow(); // remove  the shadow the coordinates are screen coordinates
        void remove_shadow(int x_from, int y_from, int x_to, int y_to); // remove (a part of) the shadow the coordinates are screen coordinates

        static int delete_its_manager;          // if true, last object deletes it's object manager on exit this is set by the first object if WHAM was not yet present
        WinSystem *its_manager;
        WinSystem *Child_manager;
        int frozen;

        Wobject *attached;
        int corner_this,corner_that,x_offset,y_offset;
        void move_to_attachpoint();


        // boxlist stuff
        Boxlist *my_boxlist;

        // deletes old boxlist(if any) and creates up to date new one
        void re_init_boxlist();

        Stack *belongings; // objects that belong to this one (like scrollbars etc)

        void (*destruct_callback)(Wobject *w);
        
};

// this is put here so the user has acces to it, not because my classes need it
#include "secretar.h"



extern WinSystem *WHAM; // this is the object manager
#endif  



